#include "StdAfx.h"
#include "Reply.h"
#include "instancechecker.h"
#include "updateutil.h"
#include "AgentWrapper.h"
#include "RxHelper.h"

const TCHAR ERROR_TAG[] = _T("error");
const TCHAR	ID_TAG[] = _T("id");
const TCHAR MESSAGE_TAG[] = _T("message");

const TCHAR LINE_FEED_STRING[] = _T("\r\n");
void CReplyBase::AppendXMLrootHeaderEx(LPTSTR szResult, int nBuflen)
{
	StringCchCat(szResult, nBuflen, _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>") );
	StringCchCat(szResult, nBuflen, LINE_FEED_STRING );
	StringCchCat(szResult, nBuflen, _T("<root>") );
	StringCchCat(szResult, nBuflen, LINE_FEED_STRING );
}

void CReplyBase::AppendXMLrootFooterEx(LPTSTR szResult, int nBuflen)
{
	StringCchCat(szResult, nBuflen, LINE_FEED_STRING );
	StringCchCat(szResult, nBuflen, _T("</root>") );
}

void CReplyBase::AppendTagEx(LPCTSTR szTagName, LPCTSTR szTagText, LPTSTR szResult, int nBuflen)
{
	CAtlString strXML;
	strXML.Format(_T("<%s>%s</%s>"), szTagName, szTagText, szTagName);

	StringCchCat(szResult, nBuflen, (LPCTSTR)strXML);
}

void CReplyBase::AppendLineFeedEx(LPTSTR szResult, int nBuflen)
{
	StringCchCat(szResult, nBuflen, LINE_FEED_STRING );
}

const TCHAR TEXT_TRUE[] = _T("true");
const TCHAR TEXT_FALSE[] = _T("false");
BOOL CReplyBase::MakeBoolResponseXMLEx(LPCTSTR szQeuryID, BOOL bReponse, LPTSTR szResult, int nBuflen, 
									   NAVERAPP_QUERY_ERROR nErrorCode, LPCTSTR ErrMsg) // NS
{
	AppendXMLrootHeaderEx(szResult, nBuflen);

	AppendTagEx(szQeuryID, bReponse ? TEXT_TRUE : TEXT_FALSE, szResult, nBuflen);

	if ( nErrorCode != NQUERY_ERROR_NONE && ErrMsg != NULL )
	{
		AppendLineFeedEx(szResult, nBuflen);

		TCHAR szBuffer[MAX_PATH] = { 0 };
		TCHAR szID[64] = { 0 };
		StringCchPrintf(szID, 64, _T("%d"), nErrorCode);
		AppendTagEx(ID_TAG, szID, szBuffer, MAX_PATH);		//id
		AppendLineFeedEx(szBuffer, MAX_PATH);
		AppendTagEx(MESSAGE_TAG, ErrMsg, szBuffer, MAX_PATH);		//message

		AppendTagEx(ERROR_TAG, szBuffer, szResult, nBuflen);
	}

	AppendXMLrootFooterEx(szResult, nBuflen);

	return TRUE;
}




//  ϴ ٿ Ѵ.
//̷ ٴ  ׷̵ ؾ .  ׻ true
BOOL CReplyAgreementShown::DoRequestDetail()
{
	MakeBoolResponseXML(m_szQueryID, g_RequestModule.GetAgent()->GetAgreementShown());

	return TRUE;
}

// ٿ  ׷̵ ֹǷ
//  Լ ׻ false ؾ .
BOOL CReplyAgreementIsShow::DoRequestDetail()
{
	MakeBoolResponseXML(m_szQueryID, g_RequestModule.GetAgent()->GetAgreementIsShow());

	return TRUE;
}

// ּҸ 
//<agreement.url>
//	<num>1</num>
//	<url.0>ڿ</url.0>
//</agreement.url>
const TCHAR NUM_TAG[] = _T("num");
const TCHAR URL0_TAG[] = _T("url.0");
BOOL CReplyAgreementURL::DoRequestDetail()
{
	AppendXMLrootHeader();

	TCHAR szAgreementURL[MAX_PATH] = { NULL };
	const BOOL bAgreementURL = g_RequestModule.GetAgent()->GetAgreementURL(szAgreementURL, _countof(szAgreementURL));

	TCHAR szBuffer[MAX_PATH] = { 0 };
	AppendTagEx(NUM_TAG, bAgreementURL? _T("1") : _T("0"), szBuffer, MAX_PATH);
	AppendLineFeedEx(szBuffer, MAX_PATH);
	AppendTagEx(URL0_TAG, szAgreementURL, szBuffer, MAX_PATH);

	AppendTag(m_szQueryID, szBuffer);

	AppendXMLrootFooter();

	return TRUE;
}

// TOOLBAR_AGENT_GUID 
// <guid>ڿ</guid>
BOOL CReplyReqGUID::DoRequestDetail()
{
	AppendXMLrootHeader();

	AppendTag(m_szQueryID, g_RequestModule.GetAgent()->GetGuid());

	AppendXMLrootFooter();
	return TRUE;
}

// Ʒ ƮŰ   ľ
// HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall\NaverToolbar
// ϸ true,  false
BOOL CReplyIsInstalled::DoRequestDetail()
{
	BOOL bExistKey = FALSE;
	if ( g_RequestModule.GetAgent()->GetIsInstalled())
	{
		bExistKey = TRUE;
	}
	else
	{
		bExistKey = FALSE;
	}

	MakeBoolResponseXML(m_szQueryID, bExistKey);

	return TRUE;
}

// ν緯 θ Ʈ о 
//<uninstaller.url>
//	<type>0</type> //	NUINSTALLER_TYPE_NSIS = 0, NUINSTALLER_TYPE_APP=1,
//	<url>ڿ</url>
//	<parameter>ڿ</parameter>
//</uninstaller.url>
const TCHAR TYPE_TAG[] = _T("type");
const TCHAR URL_TAG[] = _T("url");
const TCHAR PARAMETER_TAG[] = _T("parameter");
BOOL CReplyUninstallerURL::DoRequestDetail()
{
	AppendXMLrootHeader();

	TCHAR szUninstallFile[MAX_PATH] = { 0 };
	if ( g_RequestModule.GetAgent()->GetUninstallerFilepath(szUninstallFile, MAX_PATH ) )
	{	
		TCHAR szBuffer[MAX_PATH] = { 0 };

		TCHAR szUninstParam[MAX_PATH] = { NULL };
		int type = g_RequestModule.GetAgent()->GetUninstallerType( szUninstParam, _countof(szUninstParam));
		TCHAR szType[10] = { NULL };
		StringCchPrintf(szType, _countof(szType), _T("%d"), type);
		
		// NSIS:0 Custom:1
		AppendTagEx(TYPE_TAG, szType, szBuffer, MAX_PATH);
		AppendLineFeedEx(szBuffer, MAX_PATH);
		
		AppendTagEx(URL_TAG, szUninstallFile, szBuffer, MAX_PATH);
		AppendLineFeedEx(szBuffer, MAX_PATH);

		// ν Ķ
		AppendTagEx(PARAMETER_TAG, szUninstParam, szBuffer, MAX_PATH);

		AppendTag(m_szQueryID, szBuffer);

		AppendXMLrootFooter();

		return TRUE;			
	}
	
	return FALSE;	
}

//<version.local>ڿ</version.local>
BOOL CReplyVersionLocal::DoRequestDetail()
{
	TCHAR szVersion[MAX_PATH] = { 0 };
	MakeVersionString( szVersion, _countof(szVersion), g_RequestModule.GetAgent()->GetVersionLocal());

	AppendXMLrootHeader();
	AppendTag(m_szQueryID, szVersion);
	AppendXMLrootFooter();

	return TRUE;
}

// ֽ  ڿ ȯ
//<version.recent>ڿ</version.recent>
BOOL CReplyVersionRecent::DoRequestDetail()
{
	TCHAR szVersion[MAX_PATH] = { 0 };
	MakeVersionString( szVersion, _countof(szVersion), g_RequestModule.GetAgent()->GetVersionRecent());

	AppendXMLrootHeader();
	AppendTag(m_szQueryID, szVersion );
	AppendXMLrootFooter();
	return TRUE;
}


BOOL CReplyIsChanged::DoRequestDetail()
{
	MakeBoolResponseXML(m_szQueryID, g_RequestModule.GetAgent()->GetIsChanged() );

	return TRUE;
}

/**
	@brief AGENTDLL_UPDATE_INFO
*/
struct AGENTDLL_UPDATE_INFO
{
	NAVERAPP_UPDATE_TYPE UpdateType;
	CAgentWrapper* pUpdater;
};
DWORD WINAPI UpgradeProc(LPVOID lpData)
{
	AGENTDLL_UPDATE_INFO* pInfo = static_cast<AGENTDLL_UPDATE_INFO*>(lpData);

	CoInitialize(NULL);
	pInfo->pUpdater->DoUpgrade( (pInfo->UpdateType & NUPDATE_TYPE_SILENT) == NUPDATE_TYPE_SILENT);
	::CoUninitialize();
	delete pInfo;
	return 0;
}

// ؽ {}  ȿϹǷ Լ ° .
BOOL IsUninstalling()
{
	DECLARE_UNINSTALL_MUTEX
	if (IS_EXIST_MUTEX)
	{
		return TRUE;
	}
	else
	{
		return FALSE;
	}
}

BOOL CReplyUpgraderRun::DoRequestDetail()
{
	DECLARE_RUN_MUTEX
	if (IS_EXIST_MUTEX)
	{
		return FALSE;
	}

	TCHAR szValue[MAX_PATH] = { 0 };
	GetQueryIDnValue(m_queryXML_A, m_szQueryID, _countof(m_szQueryID), szValue, _countof(szValue));
	if ( lstrlen(szValue) == 0 )
	{
		MakeBoolResponseXML(m_szQueryID, FALSE, NQUERY_ERROR_WRONGQUERY, GetErrorString(ERROR_WRONG_QUERY) );
		return TRUE;
	}

	if ( IsUninstalling() )
	{
		MakeBoolResponseXML(m_szQueryID, FALSE, NQUERY_ERROR_ETC, GetErrorString(ERROR_UNINSTALLING) );
		return TRUE;
	}

	AGENTDLL_UPDATE_INFO* pInfo = new AGENTDLL_UPDATE_INFO;
	pInfo->pUpdater = g_RequestModule.GetAgent();
	pInfo->UpdateType = static_cast<NAVERAPP_UPDATE_TYPE>(_ttoi(szValue));

	DWORD dwThreadID;
	HANDLE hHandle = CreateThread(NULL, 0, UpgradeProc, (LPVOID)pInfo, 0, &dwThreadID);
	MakeBoolResponseXML(m_szQueryID, hHandle != NULL );

	return TRUE;
}


//Ʈ  о 
//<upgrader.status>°</upgrader.status>
//NUPDATE_STATUS_SUCCESS = 0,
//NUPDATE_STATUS_CANCEL,
//NUPDATE_STATUS_ERROR,
//NUPDATE_STATUS_STILL_ACTIVE,
//NUPDATE_STATUS_ETC_ERROR = 1000,
BOOL CReplyUpgraderStatus::DoRequestDetail()
{
	UPDATE_STATUS updateStatus = g_RequestModule.GetAgent()->GetUpdateStatus();

	AppendXMLrootHeader();

	TCHAR szID[64] = { 0 } ;
	if ( updateStatus == UPDATE_DONE )
	{
		StringCchPrintf(szID, 64, _T("%d"), NUPDATE_STATUS_SUCCESS);
		AppendTag(m_szQueryID, szID );
	}
	else if ( updateStatus == UPDATE_STARTED )
	{
		StringCchPrintf(szID, 64, _T("%d"), NUPDATE_STATUS_STILL_ACTIVE);
		AppendTag(m_szQueryID, szID );
	}
	else if ( updateStatus == UPDATE_ERROR )
	{
		TCHAR szBuffer[MAX_PATH] = { 0 };
		TCHAR szID[64] = { 0 };
		StringCchPrintf(szID, 64, _T("%d"), NUPDATE_STATUS_ETC_ERROR);
		AppendTagEx(ID_TAG, szID, szBuffer, MAX_PATH);		//id
		AppendLineFeedEx(szBuffer, MAX_PATH);
		AppendTagEx(MESSAGE_TAG, GetErrorString(g_RequestModule.GetAgent()->GetErrorCode()), szBuffer, MAX_PATH);
		AppendTag(ERROR_TAG, szBuffer);
	}
	else
	{
		ATLASSERT(FALSE);
		StringCchPrintf(szID, 64, _T("%d"), NUPDATE_STATUS_SUCCESS);
		AppendTag(m_szQueryID, szID );
	}


	AppendXMLrootFooter();

	return TRUE;
}


BOOL CReplyInstallError::DoRequestDetail()
{
	TCHAR szError[MAX_PATH] = { 0 };
	const BOOL bError = g_RequestModule.GetAgent()->GetInstallError(szError, _countof(szError));
	if ( bError && lstrlen(szError) > 0 )
	{
		MakeBoolResponseXML(m_szQueryID, TRUE, NQUERY_ERROR_ETC, szError );
	}
	else
	{
		MakeBoolResponseXML(m_szQueryID, FALSE );
	}

	return TRUE;
}


BOOL CReplyUninstallError::DoRequestDetail()
{	
	TCHAR szError[MAX_PATH] = { 0 };
	const BOOL bError = g_RequestModule.GetAgent()->GetUninstallError( szError, _countof(szError));
	if ( bError && lstrlen(szError) > 0 )
	{
		MakeBoolResponseXML(m_szQueryID, TRUE, NQUERY_ERROR_ETC, szError  );
	}
	else
	{
		MakeBoolResponseXML(m_szQueryID, FALSE );
	}

	return TRUE;
}

BOOL GetQueryIDnValue(LPCSTR szXML, LPTSTR szQeuryID, int nQueryLen, LPTSTR szValue, int nValueLen)
{
	ATLASSERT(szQeuryID != NULL);

	CRxDoc doc;
	if ( doc.ParseString( CA2T(szXML)) == FALSE )
	{
		ATLASSERT( FALSE );
		return FALSE;
	}

	StringCchCopy(szQeuryID, nQueryLen, doc.NodeAttribute(_T("root/query"), _T("id")));

	if ( szValue != NULL )
	{
		StringCchCopy(szValue, nValueLen, doc.NodeText(_T("root/query/value")));
	}

	return TRUE;
}